Comment *

 
  Console.INC  v1.0r7 - Win32 Console I/O routines, Copyright 1998 by acpizer.
 

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This program is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
   GNU General Public License for more details.

   You should have received a copy of the GNU General Public License
   along with this program; if not, write to the Free Software
   Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.


 
 Available functions in this include:

  Init_Console, Close_Console,  Write_Console, WriteConsoleChar, GetCursorPos,
  ReadConsoleChar, ReadConsoleBuff, GotoXY, SetConsoleTextColor, Clear_Console.
 


Notes:  (READ THIS)
 WriteConsole can be used only with a console screen buffer handle,
 WriteFile can be used with other handles (such as files or pipes).
 WriteConsole fails if used with a standard handle that has been redirected to
 be something other than a console handle.

 Therefore for full functionality we use WriteFile instead of WriteConsole.


 -- Please report bugs, suggestions, comments to: acpizer@unseen.org

*
;
; Initialize all our console variables and handles.
;
; Usage:
;   simply call this proc once, before you want to use the console.
;   no need to call it before every console I/O.
; -----------------------------------------------------------------------------
Align 4
Init_Console Proc
  PushA

  Call GetStdHandle, -11  ; std output handle.
  Xor  EBX, EBX           ; Error message #0
  Cmp  EAX, -1            ; check if error.
  JE   Console_Error
  Mov  [w_ConsoleHandle],EAX

  Call GetStdHandle, -10  ; std input handle.
  Xor  EBX, EBX           ; Error message #0.
  Cmp  EAX, -1            ; check if error.
  JE   Console_Error
  Mov  [r_ConsoleHandle],EAX

  Call GetConsoleMode, eax, offset s_ConsoleMode
  Xor  EBX, EBX           ; Error message #0
  Test EAX, EAX
  JZ   Console_Error

  Mov  DWord Ptr [s_ConsoleMode],EAX
  Mov  EBX, -7
  And  EAX, EBX
  Call SetConsoleMode, [r_ConsoleHandle], EAX
  Xor  EBX, EBX           ; Error message #0
  Test EAX, EAX
  JZ   Console_Error

  PopA
  Ret
Init_Console EndP

;

; Writes the given ASCIIZ string to console, no need to supply string length.
;
; Usage:
;       Call Write_Console, offset My_String
; -----------------------------------------------------------------------------
Align 4
Write_Console Proc
  PushA                    ; ESP-10
  PushF                    ; ESP-4
  Push DWord Ptr [ESP+28h] ; our string offset param.
  Call lstrlen             ; check length of ASCIIZ string.
  Mov  EBX, 4              ; Error message #4
  Test EAX, EAX
  JZ   Console_Error

  Push 0                   ; lpOverlapped
  Push offset cchWritten   ; lpcchWritten
  Push EAX                 ; Number of chars to write.
  Push DWord Ptr [ESP+34h] ; our string offset param.
  Push [w_ConsoleHandle]   ; handle of our Console-Write.
  Call WriteFile           ; boom! :)

  Mov  EBX, 1              ; Error message #1
  Test EAX, EAX
  JZ   Console_Error

  PopF
  PopA
  Ret 4                    ; Dec ESP, 4  -- restore it to old value.
EndP

;

; Writes one char to console.
;
; Usage:
;     Mov AL,mychar
;     Call WriteConsoleChar
; -----------------------------------------------------------------------------
Align 4
WriteConsoleChar Proc
  PushA
  PushF

  Mov Byte Ptr [CharBuff],AL

  Push 0                   ; lpOverlapped
  Push offset cchWritten   ; lpcchWritten
  Push 1                   ; Number of chars to write.
  Push offset CharBuff     ; lpvBuffer
  Push [w_ConsoleHandle]   ; handle of our Console-Write.
  Call WriteFile           ; boom! :)

  Mov  EBX, 1              ; Error message #1
  Test EAX, EAX
  JZ   Console_Error

  PopF
  PopA
  Ret
EndP


;
; Reads 1 char from console input, return in AL.
;
;  Usage:
;     Echo_Console_Input <ON>  or  Echo_Console_Input <OFF>
;     Call ReadConsoleChar
; -----------------------------------------------------------------------------
Align 4
ReadConsoleChar Proc
  Call ReadFile, [r_ConsoleHandle], offset TmpChar, 1, offset cchWritten, 0
  Push EBX
  Mov  EBX, 2              ; Error message #2
  Test EAX, EAX
  JZ   Console_Error
  Pop  EBX

  Mov  AL, TmpChar
  Cmp  Byte Ptr [hEcho_Console_Input],1 ; do we need to echo input?
  JNE  Dont_Echo_Console_Input
  Call WriteConsoleChar    ; write it.
Dont_Echo_Console_Input:

  Ret
EndP


;
; Reads console input into buffer.
;
; Usage:
;     Ignore_CR <ON>   or   Ignore_CR <OFF>
;     Echo_Console_Input <ON>   or   Echo_Console_Input <OFF>
;     Call ReadConsoleBuff, textbuffer, max_text_length
; -----------------------------------------------------------------------------
Align 4
ReadConsoleBuff Proc
  Mov  EBP, ESP

  Call GetConsoleScreenBufferInfo, [w_ConsoleHandle], offset Console_info
  Mov  EAX, DWord Ptr [X_Cursor]
  Mov  [Dummy_Coord], EAX

  Mov  [Console_Counter], 0

  Cmp  DWord Ptr [EBP + 8], 0
  JE   Exit_Read_Console_Loop

Read_Console_Loop:
  Mov  EAX, [EBP + 4]
  Add  EAX, [Console_Counter]
  Call ReadFile, [r_ConsoleHandle], EAX, 1, offset cchWritten, 0
  Mov  EAX, [EBP + 4]
  Add  EAX, [Console_Counter]
  Mov  AL, [EAX]
  Call WriteConsoleChar

  Mov  EAX, [EBP + 4]
  Add  EAX, [Console_Counter]
  Mov  AL,  [EAX]

  Cmp  AL, 0Dh
  JE   Exit_Read_Console_Loop

  Cmp  AL, 8
  JNE  Its_Not_a_back_space

  Cmp  [Console_Counter], 0
  JNE  Not_At_Start

  Call SetConsoleCursorPosition, [w_ConsoleHandle], DWord Ptr [Dummy_Coord]
  jmp  Read_Console_Loop

Not_At_Start:
  Dec  [Console_Counter]
;  Dec  [X_Cursor]
  Call SetConsoleCursorPosition, [w_ConsoleHandle], DWord Ptr [X_Cursor]

  Mov  EAX, [EBP + 4]
  Add  EAX, [Console_Counter]
  Mov  Byte Ptr [EAX], 0

  Mov  AL, [EAX]
  Call WriteConsoleChar

  Call SetConsoleCursorPosition, [w_ConsoleHandle], DWord Ptr [X_Cursor]
  Jmp  Read_Console_Loop

Its_Not_a_back_space:
;  Inc  [X_Cursor]
  Inc  [Console_Counter]
  Mov  EAX, [ESP + 8]
  Cmp  [Console_Counter], EAX
  JNE  Read_Console_Loop

Exit_Read_Console_Loop:
  Mov  EAX, [ESP + 4]
  Add  EAX, [Console_Counter]
  Mov  Byte Ptr [EAX], 0

  Mov  EAX, [Console_Counter]
  Ret  8
ReadConsoleBuff EndP


; Changes color attribures of characters to be written to console.
;
; Usage:
;   Call SetConsoleTextColor, your_color
; -----------------------------------------------------------------------------
Align 4
SetConsoleTextColor Proc
  PushA

  Push DWord Ptr [ESP+24h]
  Push DWord Ptr [w_ConsoleHandle]
  Call SetConsoleTextAttribute
  Push EBX
  Mov  EBX, 3          ; Error message #3
  Test EAX, EAX
  JZ   Console_Error
  Pop  EBX

  PopA
  Ret 4
EndP

; Simply clears the console buffer by filling it with blacks, and setting
; cursor to upper right corner of the screen.
;
; Usage:
;    Call Clear_Console
; -----------------------------------------------------------------------------
Clear_Console Proc
  Call GetConsoleScreenBufferInfo, [w_ConsoleHandle], offset Console_info

  Xor  EAX, EAX
  Xor  EBX, EBX
  Xor  EDX, EDX
  Mov  AX,  [Y_Console_Size]
  Mov  BX,  [X_Console_Size]
  Mul  EBX			; Calc number of chars to clear.
  Call FillConsoleOutputCharacterA,[w_ConsoleHandle], 20h, EAX, 0, offset cchWritten

  Mov  DWord Ptr [Dummy_Coord],0
  Call SetConsoleCursorPosition, [w_ConsoleHandle], [Dummy_Coord]
  Ret
Clear_Console EndP

;
;
Align 4
Console_Error Proc
  Call MessageBeep

  Push 30h
  Push offset CONSOLE_ERROR
  Push DWord Ptr [ERROR_OFFSET_TABLE + (EBX*4)]
  Push 0
  Call MessageBoxA

  Push LARGE-1
  Call ExitProcess
EndP

; Sets cursor position.
;
; Usage:
;    GotoXY your_XPos, your_XPos
; -----------------------------------------------------------------------------
GotoXY Macro XPos, YPos
  Push (YPos ShL 16) + XPos
  Push [w_ConsoleHandle]
  Call SetConsoleCursorPosition
  Mov  EBX, 5                     ; Error mesage #5
  Test EAX, EAX
  JZ   Console_Error
EndM

;

;
;Gets cursor position.
;
; Usage:
;
;   GetCursorPos
;
; returns:  X position in EAX,  Y position in EBX
; -----------------------------------------------------------------------------
GetCursorPos Macro
 Call GetConsoleScreenBufferInfo, [w_ConsoleHandle], offset Console_info
 Mov  EBX, 6
 Test EAX, EAX
 JZ   Console_Error

 MovZX EAX, Word Ptr [offset Console_info + 4] ; get the x position
 MovZX EBX, Word Ptr [offset Console_info + 6] ; get the y position
EndM

;

 ON  EQU 1
 OFF EQU 0

;

Echo_Console_Input Macro toggle_me
  Mov Byte Ptr [hEcho_Console_Input],toggle_me
EndM
;
Ignore_CR Macro toggle_me
  Mov Byte Ptr [hIgnore_CR],toggle_me
EndM

;

Close_Console Macro
 Call CloseHandle, [r_ConsoleHandle]
 Call CloseHandle, [w_ConsoleHandle]
EndM

;

 Extrn lstrlen                      :  PROC
 Extrn ReadFile                     :  PROC
 Extrn WriteFile                    :  PROC
 Extrn CloseHandle                  :  PROC
 Extrn MessageBeep                  :  PROC
 Extrn MessageBoxA                  :  PROC
 Extrn GetStdHandle                 :  PROC
 Extrn GetConsoleMode               :  PROC
 Extrn SetConsoleMode               :  PROC
 Extrn SetConsoleTextAttribute      :  PROC
 Extrn SetConsoleCursorPosition     :  PROC
 Extrn GetConsoleScreenBufferInfo   :  PROC
 Extrn FillConsoleOutputCharacterA  :  PROC

;

.Data
Copyright:    ;  remove this copyright notice and die :)
 db 0dh,0ah
 db "",0dh,0ah
 db " Win32 Assembly Console I/O Interface,  Copyright (c) 1998!, by acpizer/UCF. ",0dh,0ah
 db "",0dh,0ah

;

 CONSOLE_ERROR   db 'CONSOLE ERROR!',0
 CONSOLE_ERROR0  db ' - Error while initializing console.',0
 CONSOLE_ERROR1  db ' - Error while writing to console buffer.',0
 CONSOLE_ERROR2  db ' - Error while reading from console buffer.',0
 CONSOLE_ERROR3  db ' - Error while setting console text-color.',0
 CONSOLE_ERROR4  db ' - Zero length string, not writing it.',0
 CONSOLE_ERROR5  db ' - Unable to move cursor, check params.',0
 CONSOLE_ERROR6  db ' - Unable to get cursor position.',0
 CONSOLE_ERROR7  db ' - Some very strage Console-Terror :)',0

 Align 4

ERROR_OFFSET_TABLE:
 dd offset CONSOLE_ERROR0
 dd offset CONSOLE_ERROR1
 dd offset CONSOLE_ERROR2
 dd offset CONSOLE_ERROR3
 dd offset CONSOLE_ERROR4
 dd offset CONSOLE_ERROR5
 dd offset CONSOLE_ERROR6
 dd offset CONSOLE_ERROR7



 r_ConsoleHandle  dd ?
 w_ConsoleHandle  dd ?
 s_ConsoleMode    dd ?
 cchWritten       dd ?
 Dummy_Coord	  dd ?
 Buff_offset      dd ?
 Console_Counter  dd ?

 hEcho_Console_Input db ?
 hIgnore_CR  db ?

 Align 4
 TmpChar db ?

 Align 4
 CharBuff db ?

 Align 4
Console_info:
 X_Console_Size   dw ?
 Y_Console_Size   dw ?
 X_Cursor         dw ?
 Y_Cursor         dw ?
	          dd 5 dup (0)	; Unimportant info of console.


;
; whooooof, this was fun.
